import React, { useState, useEffect, useContext, useRef } from "react";
import { withTracker } from "meteor/react-meteor-data";
import {
  Table,
  Pagination,
  Row,
  Col,
  Select,
  ProgressBar,
} from "react-materialize";
import { Mongo } from "meteor/mongo";
import Search from "./Search";
import TableHeader from "./TableHeader";
import ColumnsButton from "./ColumnsButton";
import { AppContext } from "../contexts/appContext";
import { i18n } from "meteor/universe:i18n";
import _ from "lodash";
//import { moment } from "meteor/momentjs:moment";
import TableBody from "./TableBody";
import TablePaginator from './TablePaginator';

export const SORT_ASC = 1;
export const SORT_DESC = -1;

const DataTable = (props) => {
  const T = i18n.createComponent();

  const {
    headers,
    renderRow,
    collection,
    subsName,
    defaultInitialSort = null,
    extraFields = null, //array of string,
    refData = null,
    onChangedFulltext = null,
    //parameter s pre col leftFiler - pre mobil (small), ak nie je je 12 leftFilter aj search su potom pod sebou
    leftFilterColS = 8,
    tableClassName = "",
    searchLabel = i18n.__("messages.search") + " ...",
  } = props;

  const leftFilter = props.leftFilter || null;
  const filter = props.filter || {};
  const fulltextFields = props.fulltextFields || [];

  const fields = headers
    .filter((header) => header.field && header.field !== "action")
    .reduce((total, header) => {
      total[header.field] = 1;
      return total;
    }, {});
  if (extraFields) {
    extraFields.forEach((x) => {
      fields[x] = 1;
    });
  }

  //init data from appContext
  const { pageData, setPageData } = useContext(AppContext);
  const initialPage =
    pageData && pageData[subsName] ? pageData[subsName].page : 0;
  const initialPageSize =
    pageData && pageData[subsName] ? pageData[subsName].pageSize : 10;
  const initialFulltext =
    pageData && pageData[subsName] ? pageData[subsName].fulltext : "";
  const initialSort =
    pageData && pageData[subsName]
      ? pageData[subsName].sort || { field: "", order: SORT_ASC }
      : defaultInitialSort
      ? defaultInitialSort
      : { field: "", order: SORT_ASC };
  const initialColumns =
    pageData && pageData[subsName] ? pageData[subsName].columns : {};
  const headerColumns = headers
    ? headers
        .filter((h) => h.field && h.field !== "action")
        .reduce((accumulator, h) => {
          accumulator[h.field] = !h.hide;
          return accumulator;
        }, {})
    : {};

  //state constants
  const [sort, setSort] = useState(initialSort);
  const [fulltext, setFulltext] = useState(initialFulltext);
  const [page, setPage] = useState(initialPage);
  const [pageSize, setPageSize] = useState(initialPageSize || 10);
  const [columns, setColumns] = useState({
    ...headerColumns,
    ...initialColumns,
  });

  //   {
  //   ...initialColumns,
  //   ...headerColumns,
  // });

  const sortField = sort && sort.field ? sort.field : "";
  const sortOrder = sort && sort.order ? sort.order : SORT_ASC;

  //save data to storage
  const updateInitalData = (pPage, pSort, pFulltext, pPageSize, pColumns) => {
    const _pageData = {
      page: pPage,
      pageSize: pPageSize,
      sort: pSort,
      fulltext: pFulltext,
      columns: pColumns,
    };
    setPageData(subsName, _pageData);
    if (onChangedFulltext) {
      onChangedFulltext(pFulltext);
      setFulltext(pFulltext);
    }
  };

  const searchColS = leftFilterColS < 12 ? 12 - leftFilterColS : 12;
  const refSearch = useRef(null);
  //focus after rerender table
  useEffect(() => {
    if (refSearch.current) {
      refSearch.current.focus();
    }
  }, [fulltext]);

  const pageSizeChanged = (event) => {
    event.preventDefault();
    const _pageSize = parseInt(event.currentTarget.value);
    if (pageSize) {
      setPageSize(_pageSize);
      setPage(0);
      updateInitalData(0, sort, fulltext, _pageSize, columns);
    }
  };

  const renderSearch = () => {
    return (
      <Search
        search={fulltext}
        refSearch={refSearch}
        onSearch={(value) => {
          if (setPage) {
            setPage(0);
          }
          setFulltext(value);
          updateInitalData(0, sort, value, pageSize, columns);
        }}
        label={searchLabel}
      />
    );
  };

  const pageSizes = ["10", "25", "50", "100"];
  const renderPageSizeSelect = () => {
    return (
      <Select
        s={12}
        id={"select-pageSize"}
        name={"select-pageSize"}
        label={i18n.__("messages.show")}
        value={pageSize ? "" + pageSize : ""}
        onChange={pageSizeChanged}
      >
        {pageSizes.map((size) => (
          <option key={size} value={size}>
            {size}
          </option>
        ))}
      </Select>
    );
  };

  const renderColumnsButton = () => {
    return (
      <ColumnsButton
        headers={headers}
        columns={columns}
        setColumn={(field, checked) => {
          const _columns = { ...columns, [field]: checked };
          setColumns(_columns);
          updateInitalData(page, sort, fulltext, pageSize, _columns);
        }}
      />
    );
  };

  return (
    <React.Fragment>
      {leftFilter ? (
        <Row>
          <Col m={3} s={3} l={3}>
            {leftFilter}
          </Col>
          <Col m={2} s={3} l={2}>
            {renderPageSizeSelect()}
          </Col>
          <Col m={6} s={5} l={6}>
            {renderSearch()}
          </Col>
          {!renderRow && (
            <Col
              s={1}
              m={1}
              style={{ paddingTop: "20px", textAlign: 'right' }}
              title="Výber stĺpcov"
            >
              {renderColumnsButton()}
            </Col>
          )}
        </Row>
      ) : (
        <Row>
          <Col m={2} s={2}>
            {renderPageSizeSelect()}
          </Col>
          <Col m={9} s={9}>
            {renderSearch()}
          </Col>
          {!renderRow && (
            <Col
              s={1}
              m={1}
              style={{ paddingTop: "20px" }}
              title="Výber stĺpcov"
            >
              {renderColumnsButton()}
            </Col>
          )}
        </Row>
      )}

      <div style={{ maxWidth: "100%", overflowX: "auto" }}>
        <Table className={tableClassName || "datatable-table"}>
          <TableHeader
            headers={headers}
            columns={columns}
            sort={sort}
            onSort={(field) => {
              const newSort = { field: field, order: SORT_ASC };
              if (sort && sort.field === field) {
                if (sort.order === SORT_DESC) {
                  newSort.field = "";
                } else {
                  newSort.order = SORT_DESC;
                }
              }
              if (setPage) {
                setPage(0);
              }
              setSort(newSort);
              updateInitalData(0, newSort, fulltext, pageSize, columns);
            }}
          />
          <TableBody
            headers={headers}
            renderRow={renderRow}
            columns={columns}
            collection={collection}
            subsName={subsName}
            filter={filter}
            fulltext={fulltext}
            sortField={sortField}
            sortOrder={sortOrder}
            fields={fields}
            page={page}
            pageSize={pageSize}
          />
        </Table>
        <TablePaginator
          subsName={subsName}
          filter={filter}
          updateInitalData={updateInitalData}
          sort={sort}
          fulltext={fulltext}
          page={page}
          setPage={setPage}
          pageSize={pageSize}
          columns={columns}
        />
      </div>
    </React.Fragment>
  );
};

// const DataTable = withTracker((props) => {
//   const {
//     headers,
//     renderRow,
//     collection,
//     subsName,
//     defaultInitialSort = null,
//     extraFields = null, //array of string,
//     refData = null,
//     onChangedFulltext = null,
//   } = props;
//   const leftFilter = props.leftFilter || null;
//   const filter = props.filter || {};
//   const fulltextFields = props.fulltextFields || [];

//   const fields = headers
//     .filter((header) => header.field && header.field !== "action")
//     .reduce((total, header) => {
//       total[header.field] = 1;
//       return total;
//     }, {});
//   if (extraFields) {
//     extraFields.forEach((x) => {
//       fields[x] = 1;
//     });
//   }

//   //init data from appContext
//   const { pageData, setPageData } = useContext(AppContext);
//   const initialPage =
//     pageData && pageData[subsName] ? pageData[subsName].page : 0;
//   const initialPageSize =
//     pageData && pageData[subsName] ? pageData[subsName].pageSize : 10;
//   const initialFulltext =
//     pageData && pageData[subsName] ? pageData[subsName].fulltext : "";
//   const initialSort =
//     pageData && pageData[subsName]
//       ? pageData[subsName].sort
//       : defaultInitialSort
//       ? defaultInitialSort
//       : { field: "", order: SORT_ASC };
//   const initialColumns =
//     pageData && pageData[subsName] ? pageData[subsName].columns : {};
//   const headerColumns = headers
//     ? headers
//         .filter((h) => h.field && h.field !== "action")
//         .reduce((accumulator, h) => {
//           accumulator[h.field] = !h.hide;
//           return accumulator;
//         }, {})
//     : {};

//   //state constants
//   const [sort, setSort] = useState(initialSort);
//   const [fulltext, setFulltext] = useState(initialFulltext);
//   const [page, setPage] = useState(initialPage);
//   const [pageSize, setPageSize] = useState(initialPageSize || 10);
//   const [columns, setColumns] = useState({
//     ...headerColumns,
//     ...initialColumns,
//   });

//   //   {
//   //   ...initialColumns,
//   //   ...headerColumns,
//   // });

//   const sortField = sort && sort.field ? sort.field : "";
//   const sortOrder = sort && sort.order ? sort.order : SORT_ASC;

//   //save data to storage
//   const updateInitalData = (pPage, pSort, pFulltext, pPageSize, pColumns) => {
//     const _pageData = {
//       page: pPage,
//       pageSize: pPageSize,
//       sort: pSort,
//       fulltext: pFulltext,
//       columns: pColumns,
//     };
//     setPageData(subsName, _pageData);
//     if (onChangedFulltext) {
//       onChangedFulltext(pFulltext);
//       setFulltext(pFulltext);
//     }
//   };

//   data = {
//     dataLoading: false,
//     //rows,
//     //totalItems,
//     renderRow,
//     collection,
//     sort,
//     setSort,
//     page,
//     setPage,
//     pageSize,
//     setPageSize,
//     fulltext,
//     setFulltext,
//     columns,
//     setColumns,
//     updateInitalData,
//     onChangedFulltext,
//   };
//   if (refData) {
//     refData.current = data;
//   }

//   return { data };
// })(DataTableComponent);

export default DataTable;
